-- need to verify functionality when characters are deleted


-- known issue - when moving items to the vault, if there is already a stack of the item in the vault, the vault stack will be temporarily merged in the backpack with the stack being moved to the vault which will fire a quantity changed event for the temporary merged stack and cause a false update and possibly a false alert
-- there doesn't seem to be any effective way to detect and ignore the temporary stack.
-- I'm considering triggering the alert with a delay so the alert can be overridden in x milliseconds...

TrackingWindow = class(Turbine.UI.Lotro.Window);
local trackingBackPack;
function TrackingWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );
	self:SetText(Resource[language][35]);
	self:SetVisible(false);
	self.ShowIconsInList=true;
	self.DefaultCharLowThreshold=5;
	self.DefaultCharCriticalThreshold=1;
	self.DefaultAllLowThreshold=40;
	self.DefaultAllCriticalThreshold=8;
	self.DefaultCharHighThreshold=15;
	self.DefaultAllHighThreshold=50;
	self.NeedsQtyUpdate=false;
	self:SetOpacity(1);
	local height=405;
	local width=390; -- 8 columns
	local left=(Turbine.UI.Display:GetWidth() - width) / 2;
	local top=(Turbine.UI.Display:GetHeight() - height) / 2;
	local tmpSettings;
	tmpSettings=PatchDataLoad(Turbine.DataScope.Character, "AltInventorySettings");
	if (tmpSettings~=nil) then
		local index;
		if tmpSettings["version"]==nil then
			for index=1,#tmpSettings do
				if tmpSettings[index]~=nil then
					if (tmpSettings[index][1]=="ITTop") then
						top=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITLeft") then
						left=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITHeight") then
						height=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITWidth") then
						width=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITShowIconsInList") then
						self.ShowIconsInList=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITDefaultCharLowThreshold") then
						self.DefaultCharLowThreshold=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITDefaultCharCriticalThreshold") then
						self.DefaultCharCriticalThreshold=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="ITDefaultCharHighThreshold") then
						self.DefaultCharHighThreshold=tmpSettings[index][2];
					end
				end
			end
		else
			if tmpSettings["ITTop"]~=nil then top=tmpSettings["ITTop"] end
			if tmpSettings["ITLeft"]~=nil then left=tmpSettings["ITLeft"] end
			if tmpSettings["ITHeight"]~=nil then height=tmpSettings["ITHeight"] end
			if tmpSettings["ITWidth"]~=nil then width=tmpSettings["ITWidth"] end
			if tmpSettings["ITShowIconsInList"]~=nil then self.ShowIconsInList=tmpSettings["ITShowIconsInList"] end
			if tmpSettings["ITDefaultCharLowThreshold"]~=nil then self.DefaultCharLowThreshold=tmpSettings["ITDefaultCharLowThreshold"] end
			if tmpSettings["ITDefaultCharCriticalThreshold"]~=nil then self.DefaultCharCriticalThreshold=tmpSettings["ITDefaultCharCriticalThreshold"] end
			if tmpSettings["ITDefaultCharHighThreshold"]~=nil then self.DefaultCharHighThreshold=tmpSettings["ITDefaultCharHighThreshold"] end
		end
		-- account for changed resolution
		if left>(Turbine.UI.Display.GetWidth()-width) then left=Turbine.UI.Display.GetWidth()-width end;
		if left<0 then left=0 end;
		if width>(Turbine.UI.Display.GetWidth()-left) then width=Turbine.UI.Display.GetWidth()-left end;
		if top>(Turbine.UI.Display.GetHeight()-height) then top=Turbine.UI.Display.GetHeight()-height end;
		if top<0 then top=0 end;
		if height>(Turbine.UI.Display.GetHeight()-top) then height=Turbine.UI.Display.GetHeight()-top end;
	end
	tmpSettings=PatchDataLoad(Turbine.DataScope.Server, "AltInventorySettings");
	if (tmpSettings~=nil) then
		if tmpSettings["version"]==nil then
			local index;
			for index=1,#tmpSettings do
				if (tmpSettings[index][1]=="ITDefaultAllLowThreshold") then
					self.DefaultAllLowThreshold=tmpSettings[index][2];
				elseif (tmpSettings[index][1]=="ITDefaultAllCriticalThreshold") then
					self.DefaultAllCriticalThreshold=tmpSettings[index][2];
				elseif (tmpSettings[index][1]=="ITDefaultAllHighThreshold") then
					self.DefaultAllHighThreshold=tmpSettings[index][2];
				end
			end
		else
			if tmpSettings["ITDefaultAllLowThreshold"]~=nil then self.DefaultAllLowThreshold=tmpSettings["ITDefaultAllLowThreshold"] end
			if tmpSettings["ITDefaultAllCriticalThreshold"]~=nil then self.DefaultAllCriticalThreshold=tmpSettings["ITDefaultAllCriticalThreshold"] end
			if tmpSettings["ITDefaultAllHighThreshold"]~=nil then self.DefaultAllHighThreshold=tmpSettings["ITDefaultAllHighThreshold"] end
		end
	end

	self.TrackMenu=Turbine.UI.ContextMenu()
	self.TrackMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][46]));
	self.TrackMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][37]));

	self.QuantityMenu=Turbine.UI.ContextMenu()

	self:SetSize(width,height);
	self:SetPosition(left,top);

	self.Background=Turbine.UI.Label();
	self.Background:SetParent(self);
	self.Background:SetSize(self:GetWidth()-40,self:GetHeight()-55);
	self.Background:SetPosition(20,34)
	self.Background:SetBackColor(Turbine.UI.Color(0,0,0));

	self.CharTab=Turbine.UI.Label();
	self.CharTab:SetParent(self.Background);
	self.CharTab:SetPosition(0,0);
	self.CharTab:SetSize(65,20);
	self.CharTab:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.CharTab:SetForeColor(Turbine.UI.Color(.80,.60,.1));
	self.CharTab:SetOutlineColor(Turbine.UI.Color(.05,.05,.1));
	self.CharTab:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.CharTab:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.CharTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabUp.tga");
	self.CharTab:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
	self.CharTab:SetText(Resource[language][36]);

	self.AllTab=Turbine.UI.Label();
	self.AllTab:SetParent(self.Background);
	self.AllTab:SetPosition(self.CharTab:GetLeft()+self.CharTab:GetWidth(),self.CharTab:GetTop());
	self.AllTab:SetSize(65,20);
	self.AllTab:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.AllTab:SetForeColor(Turbine.UI.Color(.80,.60,.1));
	self.AllTab:SetOutlineColor(Turbine.UI.Color(.05,.05,.1));
	self.AllTab:SetFontStyle(Turbine.UI.FontStyle.Outline);
	self.AllTab:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.AllTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabDown.tga");
	self.AllTab:SetTextAlignment( Turbine.UI.ContentAlignment.BottomCenter);
	self.AllTab:SetText(Resource[language][29]);

	self.CharTrackPanel=Turbine.UI.Label();
	self.CharTrackPanel:SetParent(self.Background);
	self.CharTrackPanel:SetPosition(0,18);
	self.CharTrackPanel:SetSize(self:GetWidth()-40,self:GetHeight()-73);
	self.CharTrackPanel:SetVisible(true);
	self.CharTrackPanel:SetBackColor(Turbine.UI.Color(.75,.75,.75));-- make "Char" silver

	self.AllTrackPanel=Turbine.UI.Label();
	self.AllTrackPanel:SetParent(self.Background);
	self.AllTrackPanel:SetPosition(self.CharTrackPanel:GetLeft(),self.CharTrackPanel:GetTop());
	self.AllTrackPanel:SetSize(self.CharTrackPanel:GetWidth(),self.CharTrackPanel:GetHeight());
	self.AllTrackPanel:SetVisible(false);
	self.AllTrackPanel:SetBackColor(Turbine.UI.Color(.80,.60,.1));-- make "All" gold

	self.CharTab.MouseClick=function()
		self.CharTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabUp.tga");
		self.CharTab:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
		self.AllTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabDown.tga");
		self.AllTab:SetTextAlignment( Turbine.UI.ContentAlignment.BottomCenter);
		self.CharTrackPanel:SetVisible(true);
		self.AllTrackPanel:SetVisible(false);
	end
	self.AllTab.MouseClick=function()
		self.AllTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabUp.tga");
		self.AllTab:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
		self.CharTab:SetBackground("GaranStuff/AltInventory/Resources/TrackTabDown.tga");
		self.CharTab:SetTextAlignment( Turbine.UI.ContentAlignment.BottomCenter);
		self.AllTrackPanel:SetVisible(true);
		self.CharTrackPanel:SetVisible(false);
	end

	self.CharItemList=Turbine.UI.ListBox();
	self.CharItemList:SetParent(self.CharTrackPanel);
	self.CharItemList:SetPosition(1,1);
	self.CharItemList:SetSize(self.CharTrackPanel:GetWidth()-2,self.CharTrackPanel:GetHeight()-2);
	self.CharItemList:SetBackColor(Turbine.UI.Color(0,0,0));
	self.CharItemList.VScrollBar=Turbine.UI.Lotro.ScrollBar();
	self.CharItemList.VScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.CharItemList.VScrollBar:SetParent(self.CharItemList);
	self.CharItemList.VScrollBar:SetBackColor(Turbine.UI.Color(0,0,0));
	self.CharItemList.VScrollBar:SetPosition(self.CharItemList:GetWidth()-12,0)
	self.CharItemList.VScrollBar:SetWidth(12);
	self.CharItemList.VScrollBar:SetHeight(self.CharItemList:GetHeight());
	self.CharItemList.VScrollBar:SetVisible(false);
	self.CharItemList:SetVerticalScrollBar(self.CharItemList.VScrollBar);
	self.CharItemList.VScrollBar:SetZOrder(self.CharItemList:GetZOrder()+1);

	self.AllItemList=Turbine.UI.ListBox();
	self.AllItemList:SetParent(self.AllTrackPanel);
	self.AllItemList:SetPosition(1,1);
	self.AllItemList:SetSize(self.AllTrackPanel:GetWidth()-2,self.AllTrackPanel:GetHeight()-2);
	self.AllItemList:SetBackColor(Turbine.UI.Color(0,0,0));
	self.AllItemList.VScrollBar=Turbine.UI.Lotro.ScrollBar();
	self.AllItemList.VScrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.AllItemList.VScrollBar:SetParent(self.AllItemList);
	self.AllItemList.VScrollBar:SetBackColor(Turbine.UI.Color(0,0,0));
	self.AllItemList.VScrollBar:SetPosition(self.AllItemList:GetWidth()-12,0)
	self.AllItemList.VScrollBar:SetWidth(12);
	self.AllItemList.VScrollBar:SetHeight(self.AllItemList:GetHeight());
	self.AllItemList.VScrollBar:SetVisible(false);
	self.AllItemList:SetVerticalScrollBar(self.AllItemList.VScrollBar);
	self.AllItemList.VScrollBar:SetZOrder(self.AllItemList:GetZOrder()+1);

	self.CharItemArray={};
	self.CharItemIndex={};
	self.AllItemArray={};
	self.AllItemIndex={};

	self.RemoveCharacter=function(sender,charName)
		-- remove all of the tracking info for the specified character.
		local itemIndex,data;
		for itemIndex,data in ipairs(self.AllItemArray) do
			for charIndex,contrib in ipairs(data[3]) do
				if contrib[1]==charName then
					if contrib[2]~=nil then
						data[2]=data[2]-contrib[2];
					end
					if contrib[3]~=nil then
						data[2]=data[2]-contrib[3];
					end
					table.remove(data[3],charIndex);
					break;
				end
			end
		end
		self.ReSynchContributionQuantities();
		self.RefreshAllList();
	end
	self.ReSynchContributionQuantities=function()
		local charQty={};
		local vaultQty={};
		local itemIndex,data;
		local tmpBackpack=localPlayer:GetBackpack();
		local backpackSize=tmpBackpack:GetSize();

		for itemIndex=1,backpackSize do
			local tmpItem=tmpBackpack:GetItem(itemIndex);
			if tmpItem~=nil then
				local name,qty;
				name=tmpItem:GetName();
				qty=tmpItem:GetQuantity();
				if charQty[name]==nil then
					charQty[name]=qty;
				else
					charQty[name]=charQty[name]+qty;
				end
			end
		end
		local vault=VaultData[currentCharacterName]
		if vault~=nil then
			for itemIndex=1,#vault do
				local name=vault[itemIndex][5];
				if vaultQty[name]==nil then
					vaultQty[name]=vault[itemIndex][6];
				else
					vaultQty[name]=vaultQty[name]+vault[itemIndex][6];
				end
			end
		end
		for itemIndex,data in ipairs(self.AllItemArray) do
			local qty=0;
			local found=false;
			local removeIndex=-1;
			local name=data[1];
			for charIndex,contrib in ipairs(data[3]) do
				if contrib[1]==currentCharacterName then
					found=true;
					if charQty[name]==nil and vaultQty[name]==nil then
						removeIndex=charIndex;
						contrib[2]=0;
						contrib[3]=0;
					else
						if charQty[name]==nil then
							contrib[2]=0;
						else
							contrib[2]=charQty[name];
						end
						if vaultQty[name]==nil then
							contrib[3]=0;
						else
							contrib[3]=vaultQty[name];
						end
					end
				end
				qty=qty+contrib[2]+contrib[3];
			end
			if removeIndex~=-1 then
				table.remove(data[3],removeIndex); -- we have to do this outside the iteration because table.remove causes it to skip
			end
			local char=charQty[name];
			local vault=vaultQty[name];
			if char==nil then char=0 end
			if vault==nil then vault=0 end
			if (char~=0 or vault~=0) and not found then
				data[3][#data[3]+1]={currentCharacterName,char,vault};
				qty=qty+char+vault;
			end
			data[2]=qty;
		end
	end
	self.AddItem=function(sender,itemName)
		self:CharAddItem(itemName);
		self:AllAddItem(itemName);
	end
	self.CharListIndexOf=function(sender,itemName)
		local itemIndex=self.CharItemIndex[itemName];
		if itemIndex==nil then itemIndex=0 end;
		return(itemIndex);
	end
	self.AllListIndexOf=function(sender,itemName)
		local itemIndex=self.AllItemIndex[itemName];
		if itemIndex==nil then itemIndex=0 end;
		return(itemIndex);
	end
	self.CharAddItem=function(sender,itemName)
		if self:CharListIndexOf(itemName)==0 then
			-- if the character doesn't actually have any of the item we need to get the item info from GetAllItemInfo, then set the qty to 0
			local tmpNode=inventoryWindow:GetCharItemInfo(itemName);
			if tmpNode==nil then
				tmpNode=inventoryWindow:GetAllItemInfo(itemName);
				if tmpNode==nil then tmpNode={} end
				tmpNode[2]=0;
				tmpNode[3]={};
			end
			tmpNode[11]=-1; -- low threshold;
			tmpNode[12]=-1; -- low threshold;
			tmpNode[14]=-1; -- high threshold;
			if tmpNode[2]>=self.DefaultCharHighThreshold then
				tmpNode[13]=-1;
			elseif tmpNode[2]>self.DefaultCharLowThreshold then
				tmpNode[13]=0;
			elseif tmpNode[2]>self.DefaultCharCriticalThreshold then
				tmpNode[13]=1;
			else
				tmpNode[13]=2;
			end
			self.CharItemArray[#self.CharItemArray+1]=tmpNode;
			self.CharItemIndex[itemName]=#self.CharItemArray;
		end
		if self:IsVisible() then
			self:RefreshCharList();
		end
	end
	self.AllAddItem=function(sender,itemName)
		if self:AllListIndexOf(itemName)==0 then
			local tmpNode=inventoryWindow:GetAllItemInfo(itemName);
			if tmpNode==nil then
				tmpNode={};
				tmpNode[2]=0;
				tmpNode[3]={currentCharacterName,0};
			end
			tmpNode[11]=-1;
			tmpNode[12]=-1;
			tmpNode[14]=-1;
			if tmpNode[2]>=self.DefaultAllHighThreshold then
				tmpNode[13]=-1;
			elseif tmpNode[2]>self.DefaultAllLowThreshold then
				tmpNode[13]=0;
			elseif tmpNode[2]>self.DefaultAllCriticalThreshold then
				tmpNode[13]=1;
			else
				tmpNode[13]=2;
			end
			self.AllItemArray[#self.AllItemArray+1]=tmpNode;
			self.AllItemIndex[itemName]=#self.AllItemArray;
		end
		if self:IsVisible() then
			self:RefreshAllList();
		end
	end
	self.RemoveItem=function(sender,itemName)
		self:CharRemoveItem(itemName);
		self:AllRemoveItem(itemName);
	end
	self.CharRemoveItem=function(sender,itemName)
		local index=self:CharListIndexOf(itemName);
		if index>0 then
			table.remove(self.CharItemArray,index);
			self.CharItemIndex[itemName]=nil;
		end
		if self:IsVisible() then
			self:RefreshCharList();
		end
	end
	self.AllRemoveItem=function(sender,itemName)
		local index=self:AllListIndexOf(itemName);
		if index>0 then
			table.remove(self.AllItemArray,index);
			self.AllItemIndex[itemName]=nil;
		end
		if self:IsVisible() then
			self:RefreshAllList();
		end
	end
	self.RefreshCharList=function()
		-- sort list
		table.sort(self.CharItemArray,function(arg1,arg2) if arg1[13]>arg2[13] then return(true) elseif arg1[13]==arg2[13] and arg1[1]<arg2[1] then return(true) end end);
		-- clear the display
		while self.CharItemList:GetItemCount()>0 do
			-- remove the event handler so garbage collection can collect the controls...
			self.CharItemList:GetItem(1).Label.MouseClick=nil;
			self.CharItemList:RemoveItemAt(1);
		end
		self.CharItemIndex={}; -- have to rebuild the index since we just hosed it by sorting the array
		if #self.CharItemArray>0 then
			local index;
			for index=1,#self.CharItemArray do
				self.CharItemIndex[self.CharItemArray[index][1]]=index;
				tmpItem=Turbine.UI.Label();
				tmpItem:SetParent(self.CharItemList);
				tmpItem:SetPosition(0,0);
				tmpItem:SetSize(self.CharItemList:GetWidth(),20);
				tmpItem.Label=Turbine.UI.Label();
				tmpItem.Label:SetParent(tmpItem);
				tmpItem.Label:SetFontStyle(Turbine.UI.FontStyle.Outline);
				tmpItem.Label:SetOutlineColor(Turbine.UI.Color(0,0,0));

				if self.CharItemArray[index][13]==-1 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(0,.2,0));
				elseif self.CharItemArray[index][13]==0 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(0,0,0));
				elseif self.CharItemArray[index][13]==1 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(.2,.15,0));
				else
					tmpItem.Label:SetBackColor(Turbine.UI.Color(.2,0,0));
				end
				if self.ShowIconsInList then
					tmpItem.ItemIconBack=Turbine.UI.Label();
					tmpItem.ItemIconBack:SetParent(tmpItem);
					tmpItem.ItemIconBack:SetSize(32,32);
					tmpItem.ItemIconBack:SetPosition(0,0);
					tmpItem.ItemIconBack:SetBackground(self.CharItemArray[index][4]);
					tmpItem.ItemIconBack:SetOpacity(1);
					tmpItem.ItemIcon=Turbine.UI.Label();
					tmpItem.ItemIcon:SetParent(tmpItem);
					tmpItem.ItemIcon:SetSize(32,32);
					tmpItem.ItemIcon:SetPosition(0,0);
					tmpItem.ItemIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
					tmpItem.ItemIcon:SetBackground(self.CharItemArray[index][5]);
					tmpItem.ItemIcon:SetZOrder(tmpItem.ItemIconBack:GetZOrder()+1);
					tmpItem.Label:SetPosition(33,0);
					tmpItem.Label:SetSize(self.CharItemList:GetWidth(),32);
					tmpItem:SetSize(self.CharItemList:GetWidth(),32);
					if self.CharItemArray[index][2]~=nil and self.CharItemArray[index][2]>1 then
						tmpItem.QtyText=Turbine.UI.Label();
						tmpItem.QtyText:SetParent(tmpItem);
						tmpItem.QtyText:SetSize(34,27);
						tmpItem.QtyText:SetPosition(-2,0);
						tmpItem.QtyText:SetFontStyle(Turbine.UI.FontStyle.Outline);
						tmpItem.QtyText:SetOutlineColor(Turbine.UI.Color(0,0,0));
						tmpItem.QtyText:SetForeColor(Turbine.UI.Color(1,1,1));
						tmpItem.QtyText:SetTextAlignment( Turbine.UI.ContentAlignment.BottomRight );
						if self.CharItemArray[index][2]<10000 then
							tmpItem.QtyText:SetFont(Turbine.UI.Lotro.Font.Verdana12);
						else
							tmpItem.QtyText:SetFont(Turbine.UI.Lotro.Font.Verdana10);
						end
						tmpItem.QtyText:SetText(self.CharItemArray[index][2].." ");
						tmpItem.QtyText:SetZOrder(tmpItem.ItemIcon:GetZOrder()+1);
					end
				else
					tmpItem.Label:SetPosition(0,0);
					tmpItem.Label:SetSize(self.CharItemList:GetWidth(),20);
				end
				if self.CharItemArray[index][10]==Turbine.Gameplay.ItemQuality.Common then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.95,.95,.95));
				elseif self.CharItemArray[index][10]==Turbine.Gameplay.ItemQuality.Incomparable then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(0,.8,.75));
				elseif self.CharItemArray[index][10]==Turbine.Gameplay.ItemQuality.Legendary then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.8,.5,.2));
				elseif self.CharItemArray[index][10]==Turbine.Gameplay.ItemQuality.Rare then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.75,0,.75));
				elseif self.CharItemArray[index][10]==Turbine.Gameplay.ItemQuality.Uncommon then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.75,.85,.1));
				end
				tmpItem.Label:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
				tmpItem.Label:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
				tmpItem.Label:SetText(self.CharItemArray[index][1].." "..self.CharItemArray[index][3][1][2].." ("..self.CharItemArray[index][3][1][3]..")");
				tmpItem.Label.MouseClick=function(sender,args)
					if args.Button==2 then
						self.TrackMenu:GetItems():Get(1).Click=function()
							self:RemoveItem(self.CharItemArray[index][1]);
						end
						self.TrackMenu:GetItems():Get(2).Click=function()
							customThresholdWindow:SetItem(self.CharItemArray[index][1],index,false,self.CharItemArray[index][14],self.CharItemArray[index][11],self.CharItemArray[index][12]);
							customThresholdWindow:SetZOrder(self:GetZOrder()+1);
							customThresholdWindow:SetVisible(true);
						end
						self.TrackMenu:ShowMenu()
					end
				end

				self.CharItemList:AddItem(tmpItem);
			end
			if self.ShowIconsInList then
				if self.CharItemList:GetItemCount()*32>self.CharItemList:GetHeight() then
					self.CharItemList.VScrollBar:SetVisible(true);
				end
			else
				if self.CharItemList:GetItemCount()*20>self.CharItemList:GetHeight() then
					self.CharItemList.VScrollBar:SetVisible(true);
				end
			end
		end
	end
	self.RefreshAllList=function()
		local index;
		table.sort(self.AllItemArray,function(arg1,arg2) if arg1[13]>arg2[13] then return(true) elseif arg1[13]==arg2[13] and arg1[1]<arg2[1] then return(true) end end);
		while self.AllItemList:GetItemCount()>0 do
			self.AllItemList:GetItem(1).Label.MouseEnter=nil;
			self.AllItemList:GetItem(1).Label.Update=nil;
			self.AllItemList:GetItem(1).Label.MouseLeave=nil;
			self.AllItemList:GetItem(1).Label.MouseClick=nil;
			self.AllItemList:RemoveItemAt(1);
		end
		self.AllItemIndex={}; --rebuild the index
		if #self.AllItemArray>0 then
			for index=1,#self.AllItemArray do
				self.AllItemIndex[self.AllItemArray[index][1]]=index;
				local tmpItem=Turbine.UI.Label();
				tmpItem:SetParent(self.AllItemList);
				tmpItem:SetPosition(0,0);
				tmpItem:SetSize(self.AllItemList:GetWidth(),20);
				tmpItem.Label=Turbine.UI.Label();
				tmpItem.Label:SetParent(tmpItem);
				tmpItem.Label:SetFontStyle(Turbine.UI.FontStyle.Outline);
				tmpItem.Label:SetOutlineColor(Turbine.UI.Color(0,0,0));
				if self.AllItemArray[index][13]==-1 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(0,.2,0));
				elseif self.AllItemArray[index][13]==0 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(0,0,0));
				elseif self.AllItemArray[index][13]==1 then
					tmpItem.Label:SetBackColor(Turbine.UI.Color(.2,.15,0));
				else
					tmpItem.Label:SetBackColor(Turbine.UI.Color(.2,0,0));
				end
				if self.ShowIconsInList then
					tmpItem.ItemIconBack=Turbine.UI.Label();
					tmpItem.ItemIconBack:SetParent(tmpItem);
					tmpItem.ItemIconBack:SetSize(32,32);
					tmpItem.ItemIconBack:SetPosition(0,0);
					tmpItem.ItemIconBack:SetBackground(self.AllItemArray[index][4]);
					tmpItem.ItemIconBack:SetOpacity(1);
					tmpItem.ItemIcon=Turbine.UI.Label();
					tmpItem.ItemIcon:SetParent(tmpItem);
					tmpItem.ItemIcon:SetSize(32,32);
					tmpItem.ItemIcon:SetPosition(0,0);
					tmpItem.ItemIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
					tmpItem.ItemIcon:SetBackground(self.AllItemArray[index][5]);
					tmpItem.ItemIcon:SetZOrder(tmpItem.ItemIconBack:GetZOrder()+1);
					tmpItem.Label:SetPosition(33,0);
					tmpItem.Label:SetSize(self.AllItemList:GetWidth(),32);
					tmpItem:SetSize(self.AllItemList:GetWidth(),32);
					if self.AllItemArray[index][2]~=nil and self.AllItemArray[index][2]>1 then
						tmpItem.QtyText=Turbine.UI.Label();
						tmpItem.QtyText:SetParent(tmpItem);
						tmpItem.QtyText:SetSize(34,27);
						tmpItem.QtyText:SetPosition(-2,0);
						tmpItem.QtyText:SetFontStyle(Turbine.UI.FontStyle.Outline);
						tmpItem.QtyText:SetOutlineColor(Turbine.UI.Color(0,0,0));
						tmpItem.QtyText:SetForeColor(Turbine.UI.Color(1,1,1));
						tmpItem.QtyText:SetTextAlignment( Turbine.UI.ContentAlignment.BottomRight );
						if self.AllItemArray[index][2]<10000 then
							tmpItem.QtyText:SetFont(Turbine.UI.Lotro.Font.Verdana12);
						else
							tmpItem.QtyText:SetFont(Turbine.UI.Lotro.Font.Verdana10);
						end
						tmpItem.QtyText:SetText(self.AllItemArray[index][2].." ");
						tmpItem.QtyText:SetZOrder(tmpItem.ItemIcon:GetZOrder()+1);
					end
				else
					tmpItem.Label:SetPosition(0,0);
					tmpItem.Label:SetSize(self.AllItemList:GetWidth(),20);
				end
				if self.AllItemArray[index][10]==Turbine.Gameplay.ItemQuality.Common then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.95,.95,.95));
				elseif self.AllItemArray[index][10]==Turbine.Gameplay.ItemQuality.Incomparable then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(0,.8,.75));
				elseif self.AllItemArray[index][10]==Turbine.Gameplay.ItemQuality.Legendary then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.8,.5,.2));
				elseif self.AllItemArray[index][10]==Turbine.Gameplay.ItemQuality.Rare then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.75,0,.75));
				elseif self.AllItemArray[index][10]==Turbine.Gameplay.ItemQuality.Uncommon then
					tmpItem.Label:SetForeColor(Turbine.UI.Color(.75,.85,.1));
				end
				tmpItem.Label:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
				tmpItem.Label:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
				tmpItem.Label:SetText(self.AllItemArray[index][1].." "..self.AllItemArray[index][2]);

-- why the elaborate Update handler instead of using .Hover ?

				tmpItem.Label.MouseEnter=function(sender,args)
					self.ItemHoverTimer=Turbine.Engine.GetGameTime()+.7;
					tmpItem.Label:SetWantsUpdates(true)
				end
				tmpItem.Label.Update=function(sender,args)
					if Turbine.Engine.GetGameTime()>self.ItemHoverTimer then
						tmpItem.Label:SetWantsUpdates(false);
						local tmpMenu=self.QuantityMenu:GetItems()
						local charIndex,data;
						local menuItem=nil;
						tmpMenu:Clear();
						local menuList={};
						for charIndex,data in ipairs(self.AllItemArray[index][3]) do
							if data[1]=="SHARED" then
								if data[2]~=nil and data[2]~=0 then
									menuList[#menuList+1]={data[1],data[2]};
								end
							else
								if data[2]~=nil and data[2]~=0 then
									menuList[#menuList+1]={data[1],data[2]};
								end
								if data[3]~=nil and data[3]~=0 then
									menuList[#menuList+1]={data[1].." (V)",data[3]};
								end
							end
						end
						table.sort(menuList, function(arg1,arg2) if arg1[1]=="SHARED" then return true elseif arg2[1]=="SHARED" then return false elseif arg1[1]<arg2[1] then return true end end)
						local menuIndex;
						for menuIndex=1,#menuList do
							local tmpMenuItem=Turbine.UI.MenuItem();
							if menuList[menuIndex][1]=="SHARED" then
								tmpMenuItem:SetText(Resource[language][344].." "..menuList[menuIndex][2])
							else
								tmpMenuItem:SetText(menuList[menuIndex][1].." "..menuList[menuIndex][2])
							end
							tmpMenuItem:SetEnabled(false);
							tmpMenu:Add(tmpMenuItem);
						end
						self.QuantityMenu:ShowMenu();
					end
				end
				tmpItem.Label.MouseLeave=function(sender,args)
					tmpItem.Label:SetWantsUpdates(false)
				end

				tmpItem.Label.MouseClick=function(sender,args)
					if args.Button==2 then
						self.TrackMenu:GetItems():Get(1).Click=function()
							self:RemoveItem(self.AllItemArray[index][1]);
						end
						self.TrackMenu:GetItems():Get(2).Click=function()
							customThresholdWindow:SetItem(self.AllItemArray[index][1],index,true,self.AllItemArray[index][11],self.AllItemArray[index][12]);
							customThresholdWindow:SetZOrder(self:GetZOrder()+1);
							customThresholdWindow:SetVisible(true);
						end
						self.TrackMenu:ShowMenu()
					else
						local tmpMenu=self.QuantityMenu:GetItems()
						local tmpMenuItem=nil;
						local charIndex,data;
						local menuItem=nil;
						tmpMenu:Clear();
						local menuList={};
						for charIndex,data in ipairs(self.AllItemArray[index][3]) do
							if data[1]=="SHARED" then
								if data[2]~=nil and data[2]~=0 then
									menuList[#menuList+1]={data[1],data[2]};
								end
							else
								if data[2]~=nil and data[2]~=0 then
									menuList[#menuList+1]={data[1],data[2]};
								end
								if data[3]~=nil and data[3]~=0 then
									menuList[#menuList+1]={data[1].." (V)",data[3]};
								end
							end
						end
						table.sort(menuList, function(arg1,arg2) if arg1[1]=="SHARED" then return true elseif arg2[1]=="SHARED" then return false elseif arg1[1]<arg2[1] then return true end end)
						local menuIndex;
						for menuIndex=1,#menuList do
							local tmpMenuItem=Turbine.UI.MenuItem();
							if menuList[menuIndex][1]=="SHARED" then
								tmpMenuItem:SetText(Resource[language][344].." "..menuList[menuIndex][2])
							else
								tmpMenuItem:SetText(menuList[menuIndex][1].." "..menuList[menuIndex][2])
							end
							tmpMenuItem:SetEnabled(false);
							tmpMenu:Add(tmpMenuItem);
						end
						self.QuantityMenu:ShowMenu();
					end
				end
				self.AllItemList:AddItem(tmpItem);
			end
			if self.ShowIconsInList then
				if self.AllItemList:GetItemCount()*32>self.AllItemList:GetHeight() then
					self.AllItemList.VScrollBar:SetVisible(true);
				end
			else
				if self.AllItemList:GetItemCount()*20>self.AllItemList:GetHeight() then
					self.AllItemList.VScrollBar:SetVisible(true);
				end
			end
		end
	end
	self.VisibleChanged=function(sender,args)
		if self:IsVisible() then
			self.ReSynchContributionQuantities();
			self:RefreshCharList();
			self:RefreshAllList();
		end
	end
	self.SetShowIconsInList=function(sender,value)
		self.ShowIconsInList=value;
		if self:IsVisible() then
			self:RefreshCharList();
			self:RefreshAllList();
		end
	end
	self.RefreshCharStatus=function()
		local index;
		for index=1,#self.CharItemArray do
			self.CharItemArray[index][13]=-1
			if self.CharItemArray[index][14]==-1 then
				if self.CharItemArray[index][2]<self.DefaultCharHighThreshold then
					self.CharItemArray[index][13]=0
				end
			else
				if self.CharItemArray[index][2]<self.CharItemArray[index][14] then
					self.CharItemArray[index][13]=0
				end
			end
			if self.CharItemArray[index][11]==-1 then
				if self.CharItemArray[index][2]<=self.DefaultCharLowThreshold then
					self.CharItemArray[index][13]=1
				end
			else
				if self.CharItemArray[index][2]<=self.CharItemArray[index][11] then
					self.CharItemArray[index][13]=1
				end
			end
			if self.CharItemArray[index][12]==-1 then
				if self.CharItemArray[index][2]<=self.DefaultCharCriticalThreshold then
					self.CharItemArray[index][13]=2
				end
			else
				if self.CharItemArray[index][2]<=self.CharItemArray[index][12] then
					self.CharItemArray[index][13]=2
				end
			end
		end
	end
	self.RefreshAllStatus=function()
		local index;
		for index=1,#self.AllItemArray do
			self.AllItemArray[index][13]=-1
			if self.AllItemArray[index][14]==-1 then
				if self.AllItemArray[index][2]<self.DefaultAllHighThreshold then
					self.AllItemArray[index][13]=0
				end
			else
				if self.AllItemArray[index][2]<self.AllItemArray[index][14] then
					self.AllItemArray[index][13]=0
				end
			end
			if self.AllItemArray[index][11]==-1 then
				if self.AllItemArray[index][2]<=self.DefaultAllLowThreshold then
					self.AllItemArray[index][13]=1
				end
			else
				if self.AllItemArray[index][2]<=self.AllItemArray[index][11] then
					self.AllItemArray[index][13]=1
				end
			end
			if self.AllItemArray[index][12]==-1 then
				if self.AllItemArray[index][2]<=self.DefaultAllCriticalThreshold then
					self.AllItemArray[index][13]=2
				end
			else
				if self.AllItemArray[index][2]<=self.AllItemArray[index][12] then
					self.AllItemArray[index][13]=2
				end
			end
		end
	end
	self.GetShowIconsInList=function()
		return(self.ShowIconsInList);
	end
	self.SetDefaultCharLowThreshold=function(sender,value)
		self.DefaultCharLowThreshold=tonumber(value);
	end
	self.GetDefaultCharLowThreshold=function()
		return(self.DefaultCharLowThreshold);
	end
	self.SetDefaultCharHighThreshold=function(sender,value)
		self.DefaultCharHighThreshold=tonumber(value);
	end
	self.GetDefaultCharHighThreshold=function()
		return(self.DefaultCharHighThreshold);
	end
	self.SetDefaultCharCriticalThreshold=function(sender,value)
		self.DefaultCharCriticalThreshold=tonumber(value);
	end
	self.GetDefaultCharCriticalThreshold=function()
		return(self.DefaultCharCriticalThreshold);
	end
	self.SetDefaultAllLowThreshold=function(sender,value)
		self.DefaultAllLowThreshold=tonumber(value);
	end
	self.GetDefaultAllLowThreshold=function()
		return(self.DefaultAllLowThreshold);
	end
	self.SetDefaultAllCriticalThreshold=function(sender,value)
		self.DefaultAllCriticalThreshold=tonumber(value);
	end
	self.GetDefaultAllCriticalThreshold=function()
		return(self.DefaultAllCriticalThreshold);
	end
	self.SetDefaultAllHighThreshold=function(sender,value)
		self.DefaultAllHighThreshold=tonumber(value);
	end
	self.GetDefaultAllHighThreshold=function()
		return(self.DefaultAllHighThreshold);
	end
 

	self.ItemChanged={};
	self.UpdateQuantity=function(sender,itemName)
		if itemName~=nil and itemName~="" then
			-- the actual update has to be done asynchronously since performing it in the middle of a stack merge operation gives incorrect quantities
			-- and the ItemRemoved event doesn't trigger from the merge (otherwise we could forget this nonsense and just process off the ItemRemoved event)
			self.NeedsQtyUpdate=true;
			self.ItemChanged[itemName]=Turbine.Engine.GetGameTime()+.05; -- delay the asynch update to account for multiple update events being called for vault interactions - this will let us catch and process just the last interaction while also minimizing impact on frame rate
			self:SetWantsUpdates(true);
		end
	end
	self.Update=function(sender,args)
		if self.NeedsQtyUpdate then
			-- we got an UpdateQuantity event and need to complete processing it
			local processItems={};
			local tmpItem,time;
			local notDone=false;
			for tmpItem,time in pairs(self.ItemChanged) do
				notDone=true;
				if tonumber(time)<=Turbine.Engine.GetGameTime() then
					processItems[#processItems+1]=tmpItem;
				end
			end
			self.NeedsQtyUpdate=notDone;
			self:SetWantsUpdates(notDone);

			local itemCount=#processItems;
			while itemCount>0 do
				local itemName=processItems[itemCount];
				self.ItemChanged[itemName]=nil; -- remove the update entry
				table.remove(processItems,itemCount);
				self:AsyncUpdateQuantity(itemName);
				itemCount=#processItems;
			end
		end
	end
	self.AsyncUpdateQuantity=function(sender,itemName)
		-- due to the bug in merging stacks and the bug with non-stackables having a quantity of 0, we have to scan the entire backpack and get a new total for this item :(
		local newQuantity=0;
		local newBagQuantity=0;
		local newVaultQuantity=0;
		local backpack=localPlayer:GetBackpack();
		local itemIndex;
		local tmpItem;
		for itemIndex=1,backpack:GetSize() do
			tmpItem=backpack:GetItem(itemIndex);
			if tmpItem~=nil and tmpItem:GetName()==itemName then
				if tmpItem:GetQuantity()==0 then
					-- handle the odd situation with non-stackables having quantity=0
					newBagQuantity=newBagQuantity+1;
				else
					newBagQuantity=newBagQuantity+tmpItem:GetQuantity();
				end
			end
		end
		local vault=VaultData[currentCharacterName]
		for itemIndex=1,#vault do
			if vault[itemIndex][5]==itemName then
				newVaultQuantity=newVaultQuantity+vault[itemIndex][6];
			end
		end

		-- this can only be valid for items in the user's own bags/vault but has to affect both Char and All
		local alert=0;
		local index=self:CharListIndexOf(itemName);
		local status=0;
		newQuantity=newBagQuantity+newVaultQuantity;
		if index>0 then
			local delta=newQuantity-tonumber(self.CharItemArray[index][2]);
			self.CharItemArray[index][2]=newQuantity;
			if self.CharItemArray[index][3][1]==nil then self.CharItemArray[index][3][1]={} end
			self.CharItemArray[index][3][1][2]=newBagQuantity;
			self.CharItemArray[index][3][1][3]=newVaultQuantity;
			if delta~=0 then
				if self.CharItemArray[index][12]==-1 then
					if newQuantity<=self.DefaultCharCriticalThreshold then
						status=2;
					end
				else
					if newQuantity<=self.CharItemArray[index][12] then
						status=2;
					end
				end
				if status==0 then
					if self.CharItemArray[index][11]==-1 then
						if newQuantity<=self.DefaultCharLowThreshold then
							status=1;
						end
					else
						if newQuantity<=self.CharItemArray[index][11] then
							status=1;
						end
					end
				end
				if status==0 then
					if self.CharItemArray[index][14]==-1 then
						if newQuantity>=self.DefaultCharHighThreshold then
							status=-1;
						end
					else
						if newQuantity>=self.CharItemArray[index][14] then
							status=-1;
						end
					end
				end
				self.CharItemArray[index][13]=status;
				if delta<0 then
					if status==2 then
						Turbine.Shell.WriteLine("ITEM TRACKING ALERT:"..self.CharItemArray[index][1].." Character Qty="..self.CharItemArray[index][2]);
						alert=2;
					end
					if status==1 then
						Turbine.Shell.WriteLine("Item Tracking Warning:"..self.CharItemArray[index][1].." Character Qty="..self.CharItemArray[index][2]);
						if alert~=2 then alert=1 end;
					end
				else
					if status==-1 then
						Turbine.Shell.WriteLine("Item Tracking Alert:"..self.CharItemArray[index][1].." Character Qty="..self.CharItemArray[index][2]);
						if alert==0 then alert=-1 end;
					end
				end
			end
			self:RefreshCharList();
		end
		status=0;
		index=self:AllListIndexOf(itemName);
		if index>0 then
			-- need to find the user's contribution to quantity
			local oldQuantity=0;
			local foundIndex=-1;
			local tmpIndex,data;
			for tmpIndex,data in ipairs(self.AllItemArray[index][3]) do
				if data[1]==currentCharacterName then
					if data[2]==nil then data[2]=0 end
					if data[3]==nil then data[3]=0 end
					oldQuantity=data[2]+data[3];
					foundIndex=tmpIndex;
					break;
				end
			end
			if newQuantity==0 and foundIndex>-1 then
				-- need to delete the sub-entry for this character since it no longer contributes. a new entry will get created if it starts contributing again.
				table.remove(self.AllItemArray[index][3],foundIndex);
			else
				if foundIndex>-1 then
					self.AllItemArray[index][3][foundIndex][2]=newBagQuantity;
					self.AllItemArray[index][3][foundIndex][3]=newVaultQuantity;
				else
					foundIndex=#self.AllItemArray[index][3]+1;
					self.AllItemArray[index][3][foundIndex]={currentCharacterName,newBagQuantity,newVaultQuantity};
				end
			end

			local delta=newQuantity-oldQuantity;
			self.AllItemArray[index][2]=self.AllItemArray[index][2]+delta;
			if delta~=0 then
				if self.AllItemArray[index][12]==-1 then
					if self.AllItemArray[index][2]<=self.DefaultAllCriticalThreshold then
						status=2;
					end
				else
					if self.AllItemArray[index][2]<=self.AllItemArray[index][12] then
						status=2;
					end
				end
				if status==0 then
					if self.AllItemArray[index][11]==-1 then
						if self.AllItemArray[index][2]<=self.DefaultAllLowThreshold then
							status=1;
						end
					else
						if self.AllItemArray[index][2]<=self.AllItemArray[index][11] then
							status=1;
						end
					end
				end
				if status==0 then
					if self.AllItemArray[index][14]==-1 then
						if self.AllItemArray[index][2]>=self.DefaultAllHighThreshold then
							status=-1;
						end
					else
						if self.AllItemArray[index][2]>=self.AllItemArray[index][14] then
							status=-1;
						end
					end
				end
				self.AllItemArray[index][13]=status;
				if delta<0 then
					if status==2 then
						Turbine.Shell.WriteLine("ITEM TRACKING ALERT:"..self.AllItemArray[index][1].." Total Qty="..self.AllItemArray[index][2]);
						alert=2
					end
					if status==1 then
						Turbine.Shell.WriteLine("Item Tracking Warning:"..self.AllItemArray[index][1].." Total Qty="..self.AllItemArray[index][2]);
						if alert~=2 then alert=1 end
					end
				else
					if status==-1 then
						Turbine.Shell.WriteLine("Item Tracking Alert:"..self.AllItemArray[index][1].." Total Qty="..self.AllItemArray[index][2]);
						if alert==0 then alert=-1 end
					end
				end
			end
			self:RefreshAllList();
		end
		if alert==1 then minWindow:Flash(Turbine.UI.Color(1,1,0)) end;
		if alert==2 then minWindow:Flash(Turbine.UI.Color(1,0,0)) end;
		if alert==-1 then minWindow:Flash(Turbine.UI.Color(0,1,0)) end;
	end

	self.SaveData=function()
		-- save the Char data as character scope
		PatchDataSave( Turbine.DataScope.Character, "AltInventoryTrackData", self.CharItemArray);
		-- save the All data as server scope
		PatchDataSave( Turbine.DataScope.Server, "AltInventoryTrackALLData", self.AllItemArray);
	end
	self.LoadData=function()
		self.CharItemArray=PatchDataLoad(Turbine.DataScope.Character, "AltInventoryTrackData");
		if self.CharItemArray==nil then
			self.CharItemArray={}
		end
		-- create the index
		self.CharItemIndex={};
		local i,v;
		for i,v in ipairs(self.CharItemArray) do
			if v[3][1]==nil then
				v[3][1]={currentCharacterName,v[2],0};
			end
			if v[3][1][2]==nil then v[3][1][2]=v[2] end
			if v[3][1][3]==nil then
				local vaultIndex;
				local vaultQty=0;
				for vaultIndex=1,#VaultData[currentCharacterName] do
					if VaultData[currentCharacterName][vaultIndex][5]==v[1] then
						vaultQty=vaultQty+VaultData[currentCharacterName][vaultIndex][6];
					end
				end
				v[3][1][3]=vaultQty;
			end
			if v[14]==nil then v[14]=-1 end
			self.CharItemIndex[v[1]]=i;
		end
		self.AllItemArray=PatchDataLoad(Turbine.DataScope.Server, "AltInventoryTrackALLData");
		if self.AllItemArray==nil then
			self.AllItemArray={}
		end
		self.AllItemIndex={};
		for i,v in ipairs(self.AllItemArray) do
			local charIndex,data
			local sharedFound=false;
			local vaultQty={};
			for charIndex,data in ipairs(v[3]) do
				if data[1]=="SHARED" then
					sharedFound=true;
					local vaultIndex;
					vaultQty[data[1]]=0;
					if SharedStorageData~=nil then
						for vaultIndex=1,#SharedStorageData do
							if SharedStorageData[vaultIndex][5]==v[1] then
								vaultQty[data[1]]=vaultQty[data[1]]+SharedStorageData[vaultIndex][6];
							end
						end
						data[2]=vaultQty[data[1]];
					end
				elseif data[3]==nil or data[3]==0 then
					local vaultIndex;
					vaultQty[data[1]]=0;
					if VaultData[data[1]]~=nil then
						for vaultIndex=1,#VaultData[data[1]] do
							if VaultData[data[1]][vaultIndex][5]==v[1] then
								vaultQty[data[1]]=vaultQty[data[1]]+VaultData[data[1]][vaultIndex][6];
							end
						end
						data[3]=vaultQty[data[1]];
					end
				end
			end
			for charIndex=1,#characterListArray do
				local charName=characterListArray[charIndex]
				if vaultQty[charName]==nil and VaultData[charName]~=nil then
					local vaultIndex;
					vaultQty[charName]=0;
					for vaultIndex=1,#VaultData[charName] do
						if VaultData[charName][vaultIndex][5]==v[1] then
							vaultQty[charName]=vaultQty[charName]+VaultData[charName][vaultIndex][6];
						end
					end
					if vaultQty[charName]~=0 then
						v[3][#v[3]+1]={charName,0,vaultQty[charName]};
					end
				end
			end
			-- now add the Shared Storage record if it doesn't exist
			if not sharedFound then
				-- add an entry for SHARED storage. this entry should always exist, even if qty=0 (it is eliminated when displayed if 0)
				local sharedQty=0;
				local sharedIndex;
				for sharedIndex=1,#SharedStorageData do
					if SharedStorageData[sharedIndex][5]==v[1] then
						sharedQty=sharedQty+SharedStorageData[sharedIndex][6];
					end
				end
				v[3][#v[3]+1]={"SHARED",sharedQty,0};
			end
			if v[14]==nil then v[14]=-1 end
			self.AllItemIndex[v[1]]=i;
		end
	end
	self.SetCustomThresholds=function(sender, highThreshold, lowThreshold, criticalThreshold, index, applyToAll)
		if index~=nil and applyToAll~=nil then
			index=tonumber(index);
			if applyToAll==true then
				if index>0 and index<=#self.AllItemArray then
					self.AllItemArray[index][11]=lowThreshold;
					self.AllItemArray[index][12]=criticalThreshold;
					self.AllItemArrat[index][14]=highThreshold;
					self:UpdateQuantity(self.AllItemArray[index][1]);
				end
			else
				if index>0 and index<=#self.CharItemArray then
					self.CharItemArray[index][11]=lowThreshold;
					self.CharItemArray[index][12]=criticalThreshold;
					self.CharItemArray[index][14]=highThreshold;
					self:UpdateQuantity(self.CharItemArray[index][1]);
				end
			end
		end
	end
	self.SetTrimColor=function()
	end
	self.SetBackColor=function()
	end
	self.SetFontColor=function()
	end
	self.SetLanguage=function()
		self:SetText(Resource[language][35]);
		self.CharTab:SetText(Resource[language][36]);
		self.AllTab:SetText(Resource[language][29]);
		self.TrackMenu:GetItems():Get(1):SetText(Resource[language][46]);
		self.TrackMenu:GetItems():Get(2):SetText(Resource[language][37]);
	end
end

-- tables AllItemArray and CharItemArray have the following structure
-- -- [1] item name
-- -- [2] item quantity
-- -- [3] table of individual quantities (only valid for AllItemArray
-- -- [3][x][1] Character Name
-- -- [3][x][2] Bag Quantity
-- -- [3][x][3] Vault Quantity
-- -- [4] icon background resource id
-- -- [5] icon image resource id
-- -- [6] icon quality resource id (not currently used)
-- -- [7] icon shadow resource id (not currently used)
-- -- [8] icon underlay image resource id (not currently used)
-- -- [9] category
-- -- [10] quality
-- -- [11] item low threshold (-1=usedefault)
-- -- [12] item critical threshold (-1=usedefault)
-- -- [13] item status (-1 [2]>=[14], 0 [2]>=[11], 1 [11]>[2]>=[12], 2 [12]>=[2])
-- -- [14] item high threshold (-1=usedefault)

