-- to do:
-- need to handle adding currencies for curr char
-- this has to use a .Update event since the barter wallet is buggy and doesn't load all of it's currencies before plugins load and doesn't fire ItemAdded for those missing currencies when they do get added:(

-- WalletData is in the format WalletData[characterName][locale][currencyName]={quantity, smallIconID, isAccountItem} -- the 'locale' value is necessary to eliminate duplication if the user changes client languages.
-- it would be nicer if there was some way of matching up the currencies between locales but there currently is no "master list" of currencies or unique ids to match
-- when displaying an item with the isAccountItem flag, pull the qty from the current playing character, not the current selected character

euroFormat=(tonumber("1,000")==1); -- redundant but harmless
locale = "en";
goldName="Gold";
dpName="Destiny Points";
commName="Commendation";
shardName="Shard";
markName="Mark";
medName="Medallion";
sealName="Seal";

if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	goldName="Gold"
	dpName="Schicksalspunkte"
	commName="Anerkennung";
	shardName="Scherbe";
	markName="Zeichen";
	medName="Medaillon";
	sealName="Siegel";
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	goldName="Or"
	dpName="Points de destin"..string.char(195)..string.char(169).."e"
	commName="Citation";
	shardName="Eclat";
	markName="Marque";
	medName="M"..string.char(195)..string.char(169).."daillon";
	sealName="Sceau";
end

function LoadData()
	local tmpPlayer=Turbine.Gameplay.LocalPlayer.GetInstance();
	local currentCharacterName=string.gsub(tmpPlayer:GetName(),"~","_");

	-- load saved settings
	WalletData=PatchDataLoad( Turbine.DataScope.Server, "AltInventory_WalletData");
	if WalletData==nil then
		WalletData={};
	end
	if WalletData[currentCharacterName]==nil then
		WalletData[currentCharacterName]={};
	end
	-- we rebuild the "current" character wallet from scratch
	WalletData[currentCharacterName][locale]={};
	if localPlayer:GetAttributes().GetMoney~=nil then
		WalletData[currentCharacterName][locale][goldName]={localPlayer:GetAttributes():GetMoney(),0x41005e9e, false};
	else
		WalletData[currentCharacterName][locale][goldName]={0,0x41005e9e, false};
	end
	if localPlayer:GetAttributes().GetDestinyPoints~=nil then
		WalletData[currentCharacterName][locale][dpName]={localPlayer:GetAttributes():GetDestinyPoints(),0x4100a683, true};
	else
		WalletData[currentCharacterName][locale][dpName]={0,0x4100a683, true};
	end
	local tmpWallet=tmpPlayer:GetWallet();
	if tmpWallet~=nil then
		local tmpIndex;
		for tmpIndex=1,tmpWallet:GetSize() do
			local tmpCurr=tmpWallet:GetItem(tmpIndex);
			WalletData[currentCharacterName][locale][tmpCurr:GetName()]={tmpCurr:GetQuantity(),tmpCurr:GetSmallImage(), tmpCurr:IsAccountItem()};
		end
	end
end
LoadData()

WalletPanel=class(Turbine.UI.Control)
function WalletPanel:Constructor()
	Turbine.UI.Control.Constructor(self)
	self:SetMouseVisible(false);
	self:SetBackColor(backColor);
	self:SetSize(200,200);
	self.characterIndex=1;
	local charIndex;
	for charIndex=1,#characterListArray do
		if characterListArray[charIndex][1]==currentCharacterName then
			self.characterIndex=charIndex;
			break;
		end
	end
	self.characterName=currentCharacterName;

	self.LocalPlayerName=string.gsub(localPlayer:GetName(),"~","_");

-- create panel controls
	self.CurrencyList=Turbine.UI.ListBox();
	self.CurrencyList:SetParent(self);
	self.CurrencyList:SetPosition(1,1);
	self.CurrencyList:SetSize(self:GetWidth()-12,self:GetHeight()-2);
	self.CurrencyVScroll=Turbine.UI.Lotro.ScrollBar();
	self.CurrencyVScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.CurrencyVScroll:SetParent(self);
	self.CurrencyVScroll:SetSize(10,self:GetHeight()-2);
	self.CurrencyVScroll:SetPosition(self:GetWidth()-12,1);
	self.CurrencyList:SetVerticalScrollBar(self.CurrencyVScroll);
	self.PlayerAttributes=localPlayer:GetAttributes();
	self.Wallet=localPlayer:GetWallet();

	self:SetVisible(false);

	self.IsValid=function()
		return true
--		return (self.characterIndex>1);
	end

	self.itemRemovedHandler=function(sender, args)
		local currency=self.Wallet:GetItem(args.Index);
		-- need to remove the old quantity changed handler
		RemoveCallback(currency,"QuantityChanged", self.walletChangedHandler[currency:GetName()]);
		self.walletChangedHandler[currency:GetName()]=nil;
		WalletData[self.LocalPlayerName][locale][currency:GetName()]=nil;
		if self.characterIndex==1 or self.characterName==self.LocalPlayerName then
			self.LoadPanelData();
		end
	end

	self.UpdateCurrency=function(currName,qty)
		WalletData[self.LocalPlayerName][locale][currName][1]=qty;
		if self.characterIndex==1 or self.characterName==self.LocalPlayerName then
			local index;
			local dispIndex;
			for index=1,self.CurrencyList:GetItemCount() do
				local tmpRow=self.CurrencyList:GetItem(index);
				if tmpRow.Name==currName then
					if currName==goldName then
						local gp,sp,cp=0,0,qty;
						gp=math.floor(cp/100000);
						cp=cp-gp*100000;
						sp=math.floor(cp/100);
						cp=cp-sp*100;
						tmpRow.GPQty:SetText(gp);
						tmpRow.SPQty:SetText(sp);
						tmpRow.CPQty:SetText(cp);
					else
						tmpRow.CurrQty:SetText(qty);
					end
					break;
				end
			end
			if self.characterIndex==1 and WalletData[self.LocalPlayerName][locale][currName][3]==false then
				-- these are non-account items that are being displayed on the "All" character view
				qty=0;
				-- need to get sum of qty
				local char,data;
				for char,data in pairs(WalletData) do
					if data[locale]~=nil and data[locale][currName]~=nil and data[locale][currName][1]~=nil then
						qty=qty+data[locale][currName][1];
					end
				end
			end
		end
	end
	if self.PlayerAttributes.GetMoney~=nil then
		self.moneyChangedHandler = AddCallback(self.PlayerAttributes, "MoneyChanged", function() self.UpdateCurrency(goldName,localPlayer:GetAttributes():GetMoney()) end);
	end
	if self.PlayerAttributes.GetDestinyPoints~=nil then
		self.dpChangedHandler = AddCallback(self.PlayerAttributes, "DestinyPointsChanged", function() self.UpdateCurrency(dpName,localPlayer:GetAttributes():GetDestinyPoints()) end);
	end
	local walletIndex;
	self.walletChangedHandler={};
	for walletIndex=1,self.Wallet:GetSize() do
		self.walletChangedHandler[self.Wallet:GetItem(walletIndex):GetName()]=AddCallback(self.Wallet:GetItem(walletIndex),"QuantityChanged", function() self.UpdateCurrency(self.Wallet:GetItem(walletIndex):GetName(),self.Wallet:GetItem(walletIndex):GetQuantity()) end);
	end

--	self.walletItemAddedHandler=AddCallback(self.Wallet,"ItemAdded", function() self.UpdateCurrency(self.Wallet:GetItem(walletIndex):GetName(),self.Wallet:GetItem(walletIndex):GetQuantity()) end);
	self.itemRemovedHandler=AddCallback(self.Wallet,"ItemRemoved", self.itemRemovedHandler)

	self.LoadPanelData=function(sender, characterName)
if inventoryWindow~=nil and inventoryWindow.PanelShowing==self.Index then
		if characterName==nil then characterName=self.characterName end
		self.characterName=characterName;
		self.CurrencyList:ClearItems();
		-- now build a temp currency list
		local tmpCurrency={};

		local tmpIndex;
		for tmpIndex=1,#characterListArray do
			if characterListArray[tmpIndex][1]==characterName then
				self.characterIndex=tmpIndex;
				break;
			end
		end

		local tmpCurr, tmpData;
		if self.characterIndex==1 then
			-- add all currencies from ALL characters to list
			local tmpChar, tmpWallet;
			for tmpChar,tmpWallet in pairs(WalletData) do
				for tmpCurr, tmpData in pairs(tmpWallet[locale]) do
					-- if the currency already exists, just add qty
					local found=false;
					local foundIndex;
					for tmpIndex=1,#tmpCurrency do
						if tmpCurrency[tmpIndex][1]==tmpCurr then
							foundIndex=tmpIndex;
							found=true;
							break;
						end
					end
					if found then
						if tmpData[3]==false then
							-- this is not an account shared currency so add the quantity
							tmpCurrency[foundIndex][2]=tmpCurrency[foundIndex][2]+tmpData[1];
						end
					else
						local tmpOrder=10;
						if tmpCurr==goldName then
							tmpOrder=1;
						elseif tmpCurr==dpName then
							tmpOrder=2;
						elseif tmpCurr==commName then
							tmpOrder=4;
						elseif tmpCurr==shardName then
							tmpOrder=5;
						elseif tmpCurr==markName then
							tmpOrder=6;
						elseif tmpCurr==medName then
							tmpOrder=7;
						elseif tmpCurr==sealName then
							tmpOrder=8;
						end
						local tmpQty=tmpData[1];
						if tmpData[3]==true then
							-- this is an account shared currency, get the value from the current wallet
							if tmpCurr==dpName then
								if localPlayer:GetAttributes().GetDestinyPoints~=nil then
									tmpQty=localPlayer:GetAttributes():GetDestinyPoints();
								end
							else
								local tmpCurrIndex;
								for tmpCurrIndex=1,self.Wallet:GetSize() do
									if self.Wallet:GetItem(tmpCurrIndex)==tmpCurr then
										tmpQty=self.Wallet:GetItem(tmpCurrIndex):GetQuantity();
										break;
									end
								end
							end
						end
						tmpCurrency[#tmpCurrency+1]={tmpCurr, tmpQty, tmpData[2], tmpOrder,tmpData[3]};
					end
				end
			end
		else
			if WalletData[characterName]~=nil and WalletData[characterName][locale]~=nil then
				for tmpCurr, tmpData in pairs(WalletData[characterName][locale]) do
					local tmpOrder=10;
					if tmpCurr==goldName then
						tmpOrder=1;
					elseif tmpCurr==dpName then
						tmpOrder=2;
					elseif tmpCurr==commName then
						tmpOrder=4;
					elseif tmpCurr==shardName then
						tmpOrder=5;
					elseif tmpCurr==markName then
						tmpOrder=6;
					elseif tmpCurr==medName then
						tmpOrder=7;
					elseif tmpCurr==sealName then
						tmpOrder=8;
					end
					local tmpQty=tmpData[1];
					if tmpData[3]==true then
						-- this is an account shared currency, get the value from the current wallet
						if tmpCurr==dpName then
							if localPlayer:GetAttributes().GetDestinyPoints~=nil then
								tmpQty=localPlayer:GetAttributes():GetDestinyPoints();
							end
						else
							local tmpCurrIndex;
							for tmpCurrIndex=1,self.Wallet:GetSize() do
								if self.Wallet:GetItem(tmpCurrIndex)==tmpCurr then
									tmpQty=self.Wallet:GetItem(tmpCurrIndex):GetQuantity();
									break;
								end
							end
						end
					end
					tmpCurrency[#tmpCurrency+1]={tmpCurr, tmpQty, tmpData[2], tmpOrder,tmpData[3]};
				end
			end
			-- need to be sure all of the account shared currencies were added (they would be left out if the selected character was last saved before any of the shared currency was gained)
			local tmpCurrIndex;
			for tmpCurrIndex=1,self.Wallet:GetSize() do
				local tmpItem=self.Wallet:GetItem(tmpCurrIndex);
				if tmpItem:IsAccountItem() then
					local found=false;
					local foundIndex;
					tmpCurr=tmpItem:GetName()
					for tmpIndex=1,#tmpCurrency do
						if tmpCurrency[tmpIndex][1]==tmpCurr then
							foundIndex=tmpIndex;
							found=true;
							break;
						end
					end
					if not found then
						local tmpOrder=10;
						if tmpCurr==commName then
							tmpOrder=4;
						elseif tmpCurr==shardName then
							tmpOrder=5;
						elseif tmpCurr==markName then
							tmpOrder=6;
						elseif tmpCurr==medName then
							tmpOrder=7;
						elseif tmpCurr==sealName then
							tmpOrder=8;
						end
						tmpCurrency[#tmpCurrency+1]={tmpCurr, tmpItem:GetQuantity(), tmpItem:GetSmallImage(), tmpOrder, tmpItem:IsAccountItem()};
					end
				end
			end
		end
		tmpCurrency[#tmpCurrency+1]={"", "", nil, 3,false}; -- create a blank line entry
		tmpCurrency[#tmpCurrency+1]={"", "", nil, 9,false}; -- create a blank line entry
		-- now sort the currency list by tmp order then name
		table.sort(tmpCurrency,function(arg1,arg2) if arg1[4]==arg2[4] and arg1[1]<arg2[1] then return (true) else if arg1[4]<arg2[4] then return(true) end end end );
		local rowWidth=self:GetWidth()-12;
		for tmpIndex=1,#tmpCurrency do
			local tmpRow=Turbine.UI.Control();
			tmpRow:SetParent(self.CurrencyList);
			tmpRow.Name=tmpCurrency[tmpIndex][1];
			if tmpCurrency[tmpIndex][1]=="" then
				tmpRow:SetSize(rowWidth,1);
				tmpRow:SetBackColor(Turbine.UI.Color(.8,.8,.9));
				tmpRow.CurrName=Turbine.UI.Control();
				tmpRow.CurrQty=Turbine.UI.Control();
				tmpRow.Icon=Turbine.UI.Control();
			else
				tmpRow:SetSize(rowWidth,18);
				tmpRow:SetBackColor(Turbine.UI.Color(0,0,0));
				tmpRow.CurrName=Turbine.UI.Label();
				tmpRow.CurrName:SetParent(tmpRow);
				tmpRow.CurrName:SetSize((rowWidth-21)/2,18);
				tmpRow.CurrName:SetFont(Turbine.UI.Lotro.Font.Verdana16);
				tmpRow.CurrName:SetText(tmpCurrency[tmpIndex][1]);
				if tmpCurrency[tmpIndex][5]==true then
					tmpRow.CurrName:SetForeColor(Turbine.UI.Color(1,.8,.2));
				end
				tmpRow.CurrName:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft);
				if tmpCurrency[tmpIndex][1]==goldName then
					local gp,sp,cp=0,0,tmpCurrency[tmpIndex][2];
					gp=math.floor(cp/100000);
					cp=cp-gp*100000;
					sp=math.floor(cp/100);
					cp=cp-sp*100;
					tmpRow.GPQty=Turbine.UI.Label();
					tmpRow.GPQty:SetParent(tmpRow);
					tmpRow.GPQty:SetSize((rowWidth-21)/2-130,18); -- 27 per icon, 30 for cp/sp (also offset 5 pixels to remove extra space on the right side of the copper icon)
					tmpRow.GPQty:SetLeft(tmpRow.CurrName:GetLeft()+tmpRow.CurrName:GetWidth());
					tmpRow.GPQty:SetFont(Turbine.UI.Lotro.Font.Verdana16);
					tmpRow.GPQty:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
					tmpRow.GPQty:SetText(tostring(gp));

					tmpRow.GPIcon=Turbine.UI.Control();
					tmpRow.GPIcon:SetParent(tmpRow);
					tmpRow.GPIcon:SetSize(27,21); -- need to find a smaller icon :(
					tmpRow.GPIcon:SetPosition(tmpRow.GPQty:GetLeft()+tmpRow.GPQty:GetWidth(),-1);
					tmpRow.GPIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
					tmpRow.GPIcon:SetBackground(0x41007e7b);

					tmpRow.SPQty=Turbine.UI.Label();
					tmpRow.SPQty:SetParent(tmpRow);
					tmpRow.SPQty:SetSize(40,18); -- 27 per icon, 30 for cp/sp
					tmpRow.SPQty:SetLeft(tmpRow.GPIcon:GetLeft()+tmpRow.GPIcon:GetWidth());
					tmpRow.SPQty:SetFont(Turbine.UI.Lotro.Font.Verdana16);
					tmpRow.SPQty:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
					tmpRow.SPQty:SetText(tostring(sp));

					tmpRow.SPIcon=Turbine.UI.Control();
					tmpRow.SPIcon:SetParent(tmpRow);
					tmpRow.SPIcon:SetSize(27,21); -- need to find a smaller icon :(
					tmpRow.SPIcon:SetPosition(tmpRow.SPQty:GetLeft()+tmpRow.SPQty:GetWidth(),-1);
					tmpRow.SPIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
					tmpRow.SPIcon:SetBackground(0x41007e7c);

					tmpRow.CPQty=Turbine.UI.Label();
					tmpRow.CPQty:SetParent(tmpRow);
					tmpRow.CPQty:SetSize(30,18); -- 27 per icon, 30 for cp/sp
					tmpRow.CPQty:SetLeft(tmpRow.SPIcon:GetLeft()+tmpRow.SPIcon:GetWidth());
					tmpRow.CPQty:SetFont(Turbine.UI.Lotro.Font.Verdana16);
					tmpRow.CPQty:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
					tmpRow.CPQty:SetText(tostring(cp));

					tmpRow.CPIcon=Turbine.UI.Control();
					tmpRow.CPIcon:SetParent(tmpRow);
					tmpRow.CPIcon:SetSize(27,21); -- need to find a smaller icon :(
					tmpRow.CPIcon:SetPosition(tmpRow.CPQty:GetLeft()+tmpRow.CPQty:GetWidth(),-1);
					tmpRow.CPIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
					tmpRow.CPIcon:SetBackground(0x41007e7d);

				else
					tmpRow.CurrQty=Turbine.UI.Label();
					tmpRow.CurrQty:SetParent(tmpRow);
					tmpRow.CurrQty:SetSize((rowWidth-21)/2,18);
					tmpRow.CurrQty:SetLeft(rowWidth-21-tmpRow.CurrQty:GetWidth());
					tmpRow.CurrQty:SetFont(Turbine.UI.Lotro.Font.Verdana16);
					tmpRow.CurrQty:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight);
					tmpRow.CurrQty:SetText(tmpCurrency[tmpIndex][2]);
					tmpRow.Icon=Turbine.UI.Control();
					tmpRow.Icon:SetParent(tmpRow);
					if tmpCurrency[tmpIndex][1]==dpName then
						tmpRow.Icon:SetSize(15,14);
						tmpRow.Icon:SetPosition(rowWidth-16,2);
					else
						tmpRow.Icon:SetSize(16,16);
						tmpRow.Icon:SetPosition(rowWidth-16,1);
					end
					if tmpCurrency[tmpIndex][3]~=nil and tmpCurrency[tmpIndex][3]>0 then
						tmpRow.Icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
						tmpRow.Icon:SetBackground(tmpCurrency[tmpIndex][3]);
					end
				end
			end
			self.CurrencyList:AddItem(tmpRow);
		end
end
	end

	self.SaveData=function()
		PatchDataSave( Turbine.DataScope.Server, "AltInventory_WalletData",WalletData);
	end

	self.RemoveCallbacks=function()
self.Update=nil
		if self.moneyChangedHandler~=nil then
			RemoveCallback(self.PlayerAttributes, "MoneyChanged", self.moneyChangedHandler);
		end
		if self.dpChangedHandler~=nil then
			RemoveCallback(self.PlayerAttributes, "DestinyPointsChanged", self.dpChangedHandler);
		end
		if self.itemRemovedHandler~=nil then
			RemoveCallback(self.Wallet, "ItemRemoved", self.itemRemovedHandler);
		end
		local walletIndex;
		for walletIndex=1, self.Wallet:GetSize() do
			local currency=self.Wallet:GetItem(walletIndex);
			RemoveCallback(currency, "QuantityChanged", self.walletChangedHandler[currency:GetName()]);
		end
		for k,v in pairs(self.walletChangedHandler) do
			self.walletChangedHandler[k]=nil;
		end
	end

	self.RemoveCharacter=function(characterName)
		WalletData[characterName]=nil;
	end

	self.SizeChanged=function(sender,args)
		local newWidth=self:GetWidth();
		local newHeight=self:GetHeight();
		self.CurrencyList:SetWidth(newWidth-12);
		self.CurrencyList:SetHeight(newHeight-2);
		local tmpIndex;
		for tmpIndex=1,self.CurrencyList:GetItemCount() do
			local tmpRow=self.CurrencyList:GetItem(tmpIndex);
			tmpRow:SetWidth(newWidth-12);
			tmpRow.CurrName:SetWidth((newWidth-33)/2);
			if tmpRow.Name==goldName then
				tmpRow.GPQty:SetWidth((newWidth-33)/2-130); -- 27 per icon, 30 for cp/sp
				tmpRow.GPQty:SetLeft(tmpRow.CurrName:GetLeft()+tmpRow.CurrName:GetWidth());
				tmpRow.GPIcon:SetLeft(tmpRow.GPQty:GetLeft()+tmpRow.GPQty:GetWidth());
				tmpRow.SPQty:SetLeft(tmpRow.GPIcon:GetLeft()+tmpRow.GPIcon:GetWidth());
				tmpRow.SPIcon:SetLeft(tmpRow.SPQty:GetLeft()+tmpRow.SPQty:GetWidth());
				tmpRow.CPQty:SetLeft(tmpRow.SPIcon:GetLeft()+tmpRow.SPIcon:GetWidth());
				tmpRow.CPIcon:SetLeft(tmpRow.CPQty:GetLeft()+tmpRow.CPQty:GetWidth());
			else
				tmpRow.CurrQty:SetWidth((newWidth-33)/2)
				tmpRow.CurrQty:SetLeft(newWidth-33-tmpRow.CurrQty:GetWidth());
				tmpRow.Icon:SetLeft(newWidth-28);
			end
		end
		self.CurrencyVScroll:SetLeft(newWidth-12);
		self.CurrencyVScroll:SetHeight(newHeight-2);
	end
	self.SetLanguage=function()
	end
	self.SetTrimColor=function()
	end
	self.SetBackgroundColor=function()
	end
	self.SetListTextColor=function()
	end
	self.SetFontColor=function()
	end
	self.SetHeadingsColor=function()
	end

-- this update function is only used because the Turbine Wallet object doesn't get initialized correctly. if that bug ever gets fixed this can become an ItemAdded event handler
	self.walletSize=self.Wallet:GetSize();
	self.Update=function()
		local size=self.Wallet:GetSize();
		if size>self.walletSize then
			local tmpCurrency, tmpWallet;
			for tmpCurrency=1,size do
				local tmpItem=self.Wallet:GetItem(tmpCurrency);
				if WalletData[self.LocalPlayerName][locale][tmpItem:GetName()]==nil then
					-- this is a new currency, need to add it
					WalletData[self.LocalPlayerName][locale][tmpItem:GetName()]={tmpItem:GetQuantity(),tmpItem:GetSmallImage(),tmpItem:IsAccountItem()};
					-- add an event handler for it
					self.walletChangedHandler[tmpItem:GetName()]=AddCallback(tmpItem,"QuantityChanged", function() self.UpdateCurrency(tmpItem:GetName(),tmpItem:GetQuantity()) end);
				end
			end
			-- now trigger an update
			if self.characterIndex==1 or self.characterName==self.LocalPlayerName then
				self.LoadPanelData();
			end
		end
		self.walletSize=size;
	end
	self:SetWantsUpdates(true);
end
